% Learn logistic regression independently per player
% Author: Jean Honorio
% E-mail: jhonorio@csail.mit.edu
function [W b] = learn_indep_logreg(X,rho,verbose);

if ~issign(X)
    error('Only +1 and -1 are accepted in dataset');
end
if ~exist('verbose','var')
    verbose = false;
end
[T N] = size(X);
W = zeros(N);
b = zeros(N,1);
for n = 1:N
    if verbose
        fprintf('.');
    end
    idx = [1:n-1 n+1:N];
    obj_fun = @(theta) LogisticLoss(theta,[X(:,idx),-ones(T,1)],X(:,n));
    theta_init = zeros(N,1);
    rho_vec = [repmat(T*rho,N-1,1); 0];
    options.order = -1;
    options.verbose = false;
    theta = L1GeneralProjection(obj_fun,theta_init,rho_vec,options);
    W(n,idx) = theta(1:end-1);
    b(n) = theta(end);
end
if verbose
    fprintf('\n');
end
