% Learn logistic regression simultaneously for all players
% Author: Jean Honorio
% E-mail: jhonorio@csail.mit.edu
function [W b] = learn_simul_logreg(X,rho,verbose);

if ~issign(X)
    error('Only +1 and -1 are accepted in dataset');
end
if ~exist('verbose','var')
    verbose = false;
end
[T N] = size(X);
obj_fun = @(theta) simul_logistic_loss(theta,X);
theta_init = zeros(N^2,1);
rho_vec = repmat([repmat(T*rho,N-1,1); 0],N,1);
options.order = -1;
options.verbose = verbose;
theta = L1GeneralProjection(obj_fun,theta_init,rho_vec,options);
W = zeros(N);
b = zeros(N,1);
pos = 0;
for n = 1:N
    idx = [1:n-1 n+1:N];
    W(n,idx) = theta(pos+1:pos+N-1);
    b(n) = theta(pos+N);
    pos = pos+N;
end

function [l g] = simul_logistic_loss(theta,X);

theta = theta(:);
[T N] = size(X);
if length(theta) ~= N^2
    error('Theta must contain N^2 parameters');
end
nxf = zeros(T,N);
pos = 0;
for n = 1:N
    idx = [1:n-1 n+1:N];
    W_n = theta(pos+1:pos+N-1);
    b_n = theta(pos+N);
    nxf(:,n) = -X(:,n).*(X(:,idx)*W_n-b_n);
    pos = pos+N;
end
NXF = max(nxf,[],2);
c = exp(nxf-repmat(NXF,1,N));
C = exp(-NXF) + sum(c,2);
d = c./repmat(C,1,N);
l = sum(NXF+log(C));
g = zeros(N^2,1);
pos = 0;
for n = 1:N
    idx = [1:n-1 n+1:N];
    dX = d(:,n).*X(:,n);
    g(pos+1:pos+N-1) = -sum(repmat(dX,1,N-1).*X(:,idx),1);
    g(pos+N) = sum(dX);
    pos = pos+N;
end
