// Author: Jean Honorio
// E-mail: jhonorio@csail.mit.edu

#include "mex.h"
#include <stdlib.h>

void mexFunction(int nargout, mxArray *argout[], int nargin, const mxArray *argin[])
{
    int frequency, N, *x, r, i, j;
    long power_2_N, count;
    double *W, *b;
	unsigned char *table, *table_ptr;

	if (nargin != 2)
        mexErrMsgTxt("Usage: mex_get_equilibria(W.',b);");
	frequency = (nargout == 2);
    if (!frequency && (nargout > 1))
        mexErrMsgTxt("Usage: count = mex_get_equilibria(W.',b);\n   [count table] = mex_get_equilibria(W.',b);");
    N = mxGetM(argin [0]);
    if (mxGetN(argin [0]) != N)
        mexErrMsgTxt("Matrix W is not square");
    W = mxGetPr(argin [0]);
    if (mxGetM(argin [1]) * mxGetN(argin [1]) != N)
        mexErrMsgTxt("Vector b is not the same dimension of W");
    b = mxGetPr(argin [1]);
    if ((x = (int *) calloc(N, sizeof(int))) == NULL)
        mexErrMsgTxt("Insufficient memory");
	if (frequency) {
		power_2_N = 1<<N;
		argout [1] = mxCreateNumericMatrix(1, power_2_N, mxUINT8_CLASS, mxREAL);
		table = (unsigned char *) mxGetData(argout[1]);
	} else
		table = NULL;
    for (r = 0; r < N; r++)
        x [r] = -3;
    r = 0;
    count = 0;
	if (frequency)
		table_ptr = table;
    while (r >= 0) {
        int *x_r = x + r;
        *x_r += 2;
        if (*x_r > 1)
            *x_r = -3;
        if (*x_r == -3)
            r--;
        else if (r == N - 1) {
            int is_equilibrium = 1;
            double *W_ptr = W;
            for (i = 0; i < N; i++) {
                double f_i = -b [i];
                int *x_ptr = x;
                for (j = 0; j < N; j++)
                    f_i += *(W_ptr++) * *(x_ptr++);
                if (x [i] * f_i < 0) {
                    is_equilibrium = 0;
                    break;
                }
            }
            if (is_equilibrium)
                count++;
			if (frequency)
				*(table_ptr++) = is_equilibrium;
        } else
            r++;
    }
    free(x);
	argout [0] = mxCreateDoubleScalar(count);
}
